library IEEE;
library work;
library unisim;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use UNISIM.VCOMPONENTS.ALL;

use work.basic_trigger_v2_pkg.ALL;
use work.bicolor_led_ctrl_pkg.ALL;

entity basic_trigger_v2_top is
   generic(g_NUMBER_OF_CHANNELS : NATURAL := work.basic_trigger_v2_pkg.c_NUMBER_OF_CHANNELS);
   port(FPGA_CLK_P           : in  STD_LOGIC;
        FPGA_CLK_N           : in  STD_LOGIC;
        FPGA_OE              : out STD_LOGIC;
        FPGA_TRIG_TTL_OE     : out STD_LOGIC;
        FPGA_INV_OE          : out STD_LOGIC;
        FPGA_BLO_OE          : out STD_LOGIC;
        TTL_or_INV_TTL_N     : in  STD_LOGIC;
        EXTRA_SWITCH         : in  STD_LOGIC_VECTOR(1 downto 1);
        MR_N                 : out STD_LOGIC;
        FPGA_INPUT_TTL_N     : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
        FPGA_OUT_TTL         : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
        FPGA_BLO_IN          : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
        FPGA_TRIG_BLO        : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
        PULSE_FRONT_LED_N    : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
        PULSE_REAR_LED       : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
        INV_IN_N             : in  STD_LOGIC_VECTOR(4 downto 1);
        INV_OUT              : out STD_LOGIC_VECTOR(4 downto 1);
        RST_N                : in  STD_LOGIC;
        CLK20_VCXO           : in  STD_LOGIC;
        LED_CTRL0            : out STD_LOGIC;
        LED_CTRL0_OEN        : out STD_LOGIC;
        LED_CTRL1            : out STD_LOGIC;
        LED_CTRL1_OEN        : out STD_LOGIC;
        LED_MULTICAST_2_0    : out STD_LOGIC;
        LED_MULTICAST_3_1    : out STD_LOGIC;
        LED_WR_GMT_TTL_TTLN  : out STD_LOGIC;
        LED_WR_LINK_SYSERROR : out STD_LOGIC;
        LED_WR_OK_SYSPW      : out STD_LOGIC;
        LED_WR_OWNADDR_I2C   : out STD_LOGIC);
end basic_trigger_v2_top;

architecture Behavioral of basic_trigger_v2_top is

   --! TODO: connect to Matthieu's LEDs IP core
   signal s_leds_array_image1  : t_leds_array;
   signal s_led_state_array    : t_led_state_array(
                                      c_NB_ARRAY_LEDS - 1 downto 0);
   signal s_pulse_i_rear       : STD_LOGIC_VECTOR(6 downto 1);
   signal s_switch             : STD_LOGIC_VECTOR(1 downto 1);

   signal s_pulse_i          : STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
   signal s_pulse_i_front    : STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
   signal s_pulse_o          : STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
   signal s_pulse_n_o        : STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
   signal s_led              : STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);

   signal s_fpga_o_en        : STD_LOGIC;
   signal s_fpga_o_ttl_en    : STD_LOGIC;
   signal s_fpga_o_inv_en    : STD_LOGIC;
   signal s_fpga_o_blo_en    : STD_LOGIC;

   signal s_rst_n              : STD_LOGIC := '0';

   signal s_level            : STD_LOGIC;

   signal s_clk_125m_i         : STD_LOGIC;
   signal s_clk_125m_prebufg   : STD_LOGIC;
   signal s_clk_125m           : STD_LOGIC;
   signal s_clk_200m_prebufg   : STD_LOGIC;
   signal s_clk_200m           : STD_LOGIC;
   signal s_clk_fb_prebufg     : STD_LOGIC;
   signal s_clk_fb             : STD_LOGIC;

   constant c_RST_CLKS       : NATURAL := 256; --! @8ns
   signal s_rst              : STD_LOGIC_VECTOR(c_RST_CLKS - 1 downto 0)
                                                            := (others => '1');
   signal s_locked           : STD_LOGIC;

begin

--   g_basic_trigger_v2_inputs: for i in 1 to 6 generate
--   begin
--      inst_125m_IBUFDS : IBUFDS
--      generic map (DIFF_TERM    => TRUE,  IBUF_LOW_PWR => TRUE)
--      port map (O  => s_pulse_i_rear(i), I  => FPGA_RS485_INA_TTL(i),
--                                         IB => FPGA_RS485_INB_TTL(i));
--   end generate;

   s_pulse_i_rear  <= not(FPGA_BLO_IN);
   s_switch        <= EXTRA_SWITCH;

   --! Level 1 means TTL_N  Switch DOWN
   --!       0 means TTL    Switch UP
   s_level         <= TTL_or_INV_TTL_N;

   --! s_level '1' means TTL input, as we have one inverter in the board we
   --! invert here.
   s_pulse_i_front <= not(FPGA_INPUT_TTL_N) when TTL_or_INV_TTL_N = '0'
                                            else FPGA_INPUT_TTL_N;
   s_pulse_i       <= s_pulse_i_front or FPGA_BLO_IN;

   FPGA_OE            <= s_fpga_o_en when s_switch(1) = '0' else '0';
   FPGA_TRIG_TTL_OE   <= s_fpga_o_ttl_en;
   FPGA_INV_OE        <= s_fpga_o_inv_en;
   FPGA_BLO_OE        <= s_fpga_o_blo_en;

   PULSE_FRONT_LED_N     <= not(s_led);--! No need of accurate sync, hence we place
   PULSE_REAR_LED        <= not(s_led);--! some combinatorial here.

   FPGA_OUT_TTL    <= s_pulse_o  when s_level = '0'
                                 else not(s_pulse_o);
   FPGA_TRIG_BLO   <= s_pulse_o;

   INV_OUT         <= INV_IN_N;--! As we have one Schmitt inverter in the input,
                               --! and a buffer in the output, there's no need
                               --! of invert here.

   U_Buf_CLK_PLL : IBUFGDS
   generic map (DIFF_TERM    => TRUE,
                IBUF_LOW_PWR => TRUE)
   port map (O  => s_clk_125m_i,                -- Buffer output
             I  => FPGA_CLK_P,
             IB => FPGA_CLK_N);

  -- set up the fabric PLL_BASE to drive the BUFPLL
  pll_base_inst : PLL_BASE
    generic map (
      BANDWIDTH          => "OPTIMIZED",
      CLK_FEEDBACK       => "CLKFBOUT",
      COMPENSATION       => "SYSTEM_SYNCHRONOUS",
      DIVCLK_DIVIDE      => 1, CLKFBOUT_MULT => 8,     CLKFBOUT_PHASE     => 0.000,
      CLKOUT0_DIVIDE     => 8, CLKOUT0_PHASE => 0.000, CLKOUT0_DUTY_CYCLE => 0.500,
      CLKOUT1_DIVIDE     => 5, CLKOUT1_PHASE => 0.000, CLKOUT1_DUTY_CYCLE => 0.500,
      CLKIN_PERIOD       => 8.0,
      REF_JITTER         => 0.010)
    port map (
      -- Output clocks
      CLKFBOUT => s_clk_fb_prebufg,
      CLKOUT0  => s_clk_125m_prebufg,
      CLKOUT1  => s_clk_200m_prebufg,
      CLKOUT2  => open,      CLKOUT3  => open,
      CLKOUT4  => open,      CLKOUT5  => open,
      -- Status and control signals
      LOCKED   => s_locked,  RST      => s_rst(c_RST_CLKS - 1),
      -- Input clock control
      CLKFBIN  => s_clk_fb,
      CLKIN    => s_clk_125m_i);

    s_clk_125m_bufg : BUFG
      port map (O => s_clk_125m, I => s_clk_125m_prebufg);

    s_clk_200m_bufg : BUFG
      port map (O => s_clk_200m, I => s_clk_200m_prebufg);

    s_clk_fb_bufg   : BUFG
      port map (O => s_clk_fb,   I => s_clk_fb_prebufg);


   i_repetitors: for i in 1 to g_NUMBER_OF_CHANNELS generate
   begin
      trigger: basic_trigger_core
      port map (
         wb_rst_i  => s_rst(c_RST_CLKS - 1),
         wb_clk_i  => s_clk_200m,

         pulse_i   => s_pulse_i(i),
         pulse_o   => s_pulse_o(i),
         pulse_n_o => s_pulse_n_o(i),

         crop_o    => open,

         led_o     => s_led(i));
   end generate i_repetitors;

   p_reset_chain : process(s_clk_125m_i) is
   begin
      if rising_edge(s_clk_125m_i) then

         s_rst(0) <= '0';
         for i in 1 to c_RST_CLKS - 1 loop
            s_rst(i) <= s_rst(i-1);
         end loop;

         if s_rst(c_RST_CLKS - 1) = '1' then
         --! First we reset the FPGA general output enable
         --! Then we let one clock delay for the rest of signals
            MR_N             <= '0';
            s_fpga_o_en      <= '0';
            s_fpga_o_ttl_en  <= '0';
            s_fpga_o_inv_en  <= '0';
            s_fpga_o_blo_en  <= '0';

         else
            MR_N             <= '1';
            s_fpga_o_en      <= '1';
            s_fpga_o_ttl_en  <= s_fpga_o_en;
            s_fpga_o_inv_en  <= s_fpga_o_en;
            s_fpga_o_blo_en  <= s_fpga_o_en;
         end if;
      end if;
   end process p_reset_chain;

   s_leds_array_image1.top.pwr <= '1';

   --! Here are organized in the same disposition as in the front panel.
   --! Take a look to basic_trigger_v2_pkg.vhd for the correct order for
   --! bicolor_led_ctrl
   s_led_state_array(c_LED_NB_PWR)    <= f_LED_STATE(c_LED_COLOR_PWR)
                                        when s_leds_array_image1.top.PWR = '1'
                                        else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_ERR)    <= f_LED_STATE(c_LED_COLOR_ERR)
                                        when s_leds_array_image1.top.ERR = '1'
                                        else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_TTL_N)  <= f_LED_STATE(c_LED_COLOR_TTL_N)
                                        when s_leds_array_image1.top.TTL_N = '1'
                                        else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_I2C)    <= f_LED_STATE(c_LED_COLOR_I2C)
                                        when s_leds_array_image1.top.I2C = '1'
                                        else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_WR_OK)   <= f_LED_STATE(c_LED_COLOR_WR_OK)
                                  when s_leds_array_image1.middle.WR_OK = '1'
                                  else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_WR_LINK) <= f_LED_STATE(c_LED_COLOR_WR_LINK)
                                  when s_leds_array_image1.middle.WR_LINK = '1'
                                  else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_WR_GMT)  <= f_LED_STATE(c_LED_COLOR_WR_GMT)
                                  when s_leds_array_image1.middle.WR_GMT = '1'
                                  else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_WR_ADDR) <= f_LED_STATE(c_LED_COLOR_WR_ADDR)
                                  when s_leds_array_image1.middle.WR_ADDR = '1'
                                  else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_MULTICAST3) <= f_LED_STATE(c_LED_COLOR_MULTICAST3)
                                  when s_leds_array_image1.bottom.MULTICAST(3) = '1'
                                  else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_MULTICAST2) <= f_LED_STATE(c_LED_COLOR_MULTICAST2)
                                  when s_leds_array_image1.bottom.MULTICAST(2) = '1'
                                  else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_MULTICAST1) <= f_LED_STATE(c_LED_COLOR_MULTICAST1)
                                  when s_leds_array_image1.bottom.MULTICAST(1) = '1'
                                  else f_LED_STATE(c_LED_OFF);
   s_led_state_array(c_LED_NB_MULTICAST0) <= f_LED_STATE(c_LED_COLOR_MULTICAST0)
                                  when s_leds_array_image1.bottom.MULTICAST(0) = '1'
                                  else f_LED_STATE(c_LED_OFF);
   s_rst_n   <= not(RST_N);

   inst_bicolor_led_ctrl: bicolor_led_ctrl
   generic map(g_NB_COLUMN    => c_NB_COLUMN,
               g_NB_LINE      => c_NB_LINE,
               g_CLK_FREQ     => 20000000,
               g_REFRESH_RATE => 250)
   port map(rst_n_i         => s_rst_n,
            clk_i           => CLK20_VCXO,
            led_intensity_i => c_LED_INTENSITY,
            led_state_i     => f_STD_LOGIC_VECTOR(s_led_state_array),
            column_o(0)     => LED_WR_OWNADDR_I2C,
            column_o(1)     => LED_WR_GMT_TTL_TTLN,
            column_o(2)     => LED_WR_LINK_SYSERROR,
            column_o(3)     => LED_WR_OK_SYSPW,
            column_o(4)     => LED_MULTICAST_2_0,
            column_o(5)     => LED_MULTICAST_3_1,
            line_o(0)       => LED_CTRL0,
            line_o(1)       => LED_CTRL1,
            line_oen_o(0)   => LED_CTRL0_OEN,
            line_oen_o(1)   => LED_CTRL1_OEN);

end Behavioral;


